@@ -15,7 +15,6 @@ __all__ = ['Banshee',
            'gnome-screensaver-dialog',
            'gnome-search-tool',
            'gnome-shell',
-           'gnome-terminal',
            'gnome-window-properties',
            'gtk-window-decorator',
            'Instantbird',
@@ -28,5 +27,7 @@ __all__ = ['Banshee',
            'planner',
            'rhythmbox',
            'soffice',
+           'SeaMonkey',
+           'smuxi-frontend-gnome',
            'Thunderbird',
            'xfwm4']
