@@ -103,7 +103,8 @@ class Utilities(web.Utilities):
             debug.println(debug.LEVEL_INFO, msg, True)
             return False
 
-        if not(role1 == role2 == pyatspi.ROLE_FRAME):
+        roles = self._topLevelRoles()
+        if not (role1 in roles and role2 in roles):
             return False
 
         try:
@@ -141,3 +142,48 @@ class Utilities(web.Utilities):
             objects.extend([x for x in root[0] if isExtra(x)])
 
         return objects
+
+    def isEditableMessage(self, obj):
+        """Returns True if this is an editable message."""
+
+        if not obj:
+            return False
+
+        if not obj.getState().contains(pyatspi.STATE_EDITABLE):
+            return False
+
+        if self.isDocument(obj):
+            msg = "GECKO: %s is believed to be an editable message" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return True
+
+        document = self.getContainingDocument(obj)
+        if document and document.getState().contains(pyatspi.STATE_EDITABLE):
+            msg = "GECKO: %s is in an editable document: %s" % (obj, document)
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return True
+
+        msg = "GECKO: Editable %s not in an editable document" % obj
+        debug.println(debug.LEVEL_INFO, msg, True)
+        return False
+
+    def isNotRealDocument(self, obj):
+        try:
+            name = obj.name
+        except:
+            msg = "GECKO: Exception getting name for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
+
+        if name.startswith("moz-extension"):
+            return True
+
+        if "self-repair.mozilla.org" in name:
+            return True
+
+        return False
+
+    def canBeActiveWindow(self, window, clearCache=False):
+        # We apparently having missing events from Gecko requiring
+        # we update the cache. This is not performant. :(
+        return super().canBeActiveWindow(window, True)
